@@ -1,35 +1,73 @@
+from loguru import logger
+import click
 import io
 import json
-import logging
-from collections import OrderedDict
+from rapidfuzz import fuzz
+from dataherb.utils.data import flatten_dict as _flatten_dict
+from dataherb.fetch.remote import get_data_from_url as _get_data_from_url
 
 import pandas as pd
 from dataherb.fetch.remote import get_data_from_url as _get_data_from_url
 from dataherb.utils.data import flatten_dict as _flatten_dict
-from fuzzywuzzy import fuzz, process
 
-
-logging.basicConfig()
-logger = logging.getLogger("dataherb.core.base")
+from dataherb.parse.model_json import MetaData
 
 
 class Herb(object):
     """
     Herb is a collection of the dataset.
     """
-    def __init__(self, herb_meta_json):
+
+    def __init__(self, meta_dict):
         """
-        :param herb: the dictionary that specifies the herb
-        :type herb: dict
+        :param herb_meta_json: the dictionary that specifies the herb
+        :type herb_meta_json: dict
         """
-        if isinstance(herb_meta_json, dict):
-            herb_meta_json = OrderedDict(herb_meta_json)
-        self.herb_meta_json = herb_meta_json
-        self.name = self.herb_meta_json.get("name")
-        self.description = self.herb_meta_json.get("description")
-        self.repository = self.herb_meta_json.get("repository")
-        self.id = self.herb_meta_json.get("id")
-        self._get_leaves()
+        if isinstance(meta_dict, dict):
+            self.herb_meta_json = meta_dict
+        elif isinstance(meta_dict, MetaData):
+            logger.debug("get herb_meta_json from MetaData ...")
+            self.herb_meta_json = meta_dict.metadata
+        else:
+            logger.debug(f"input meta type ({type(meta_dict)}) is not supported.")
+            click.BadParameter(
+                f"input meta type ({type(meta_dict)}) is not supported.",
+                param=meta_dict,
+            )
+
+        self._from_meta_dict(self.herb_meta_json)
+
+    def _from_meta_dict(self, meta_dict):
+        self.metadata_uri = meta_dict.get("metadata_uri")
+        self.datapackage = meta_dict.get("datapackage")
+        if not self.datapackage:
+            self.update_datapackage()
+        self.name = meta_dict.get("name")
+        self.description = meta_dict.get("description")
+        self.repository = meta_dict.get("repository")
+        self.id = meta_dict.get("id")
+
+    def update_datapackage(self):
+        """
+        update_datapackage gets the datapackage metadata from the metadata_uri
+        """
+
+        file_content = _get_data_from_url(self.metadata_uri)
+
+        if not file_content.status_code == 200:
+            file_error_msg = "Could not fetch remote file: {}; {}".format(
+                self.url, file_content.status_code
+            )
+            click.ClickException(file_error_msg)
+            # file_content = json.dumps([{"url": self.url, "error": file_error_msg}])
+        else:
+            file_content = file_content.json()  # .decode(self.decode)
+
+        self.datapackage_meta = file_content
+
+        self.herb_meta_json["datapackage"] = self.datapackage_meta
+
+        return self.datapackage_meta
 
     def search_score(self, keywords, keys=None):
         """
@@ -48,8 +86,7 @@ class Herb(object):
             keywords = [keywords]
 
         herb_for_search = {
-            key:val for key, val in self.herb_meta_json.items()
-            if key in keys
+            key: val for key, val in self.herb_meta_json.items() if key in keys
         }
 
         herb_for_search = _flatten_dict(herb_for_search)
@@ -66,43 +103,36 @@ class Herb(object):
 
         return max_score
 
+    @property
     def metadata(self, keys=None):
         """
         metadata formats the metadata of the herb
         """
 
-        return self.herb_meta_json
+        return self.herb_meta_json.copy()
 
     def download(self):
         """
         download downloads the dataset
         """
 
-        data_files = []
+        raise NotImplementedError("Not implemented")
 
-        for leaf_meta in self.herb_meta_json.get("data"):
-            leaf = Leaf(leaf_meta, self)
-            data_files.append(leaf.download())
+        # data_files = []
 
-        return data_files
+        # for leaf_meta in self.herb_meta_json.get("data"):
+        #     leaf = Leaf(leaf_meta, self)
+        #     data_files.append(leaf.download())
 
-    def _get_leaves(self):
-        """
-        leaf fetches the leaf/leaves of the Herb.
-        """
-        self.leaves = {}
-
-        for leaf_meta in self.herb_meta_json.get("data"):
-            leaf_meta_path = leaf_meta.get("path")
-            leaf = Leaf(leaf_meta, self)
-            self.leaves[leaf_meta_path] = leaf
+        # return data_files
 
     def __str__(self):
-        meta = self.metadata()
+        meta = self.metadata
         authors = meta.get("contributors", [])
         authors = ", ".join([author.get("name") for author in authors])
         return (
             f"DataHerb ID: {meta.get('id')}\n"
+            f"name: {meta.get('name')}"
             f"description: {meta.get('description')}\n"
             f"contributors: {authors}"
         )
@@ -110,6 +140,8 @@ class Herb(object):
 
 class Leaf(object):
     """
+    Deprecated
+
     Leaf is a data file of the Herb.
     """
 
@@ -118,8 +150,7 @@ class Leaf(object):
         self.herb = herb
 
         self.url = "https://raw.githubusercontent.com/{}/master/{}".format(
-            self.herb.repository,
-            self.leaf_meta_json.get("path")
+            self.herb.repository, self.leaf_meta_json.get("path")
         )
         self.format = self.leaf_meta_json.get("format")
         # decode the file content using decode
@@ -138,16 +169,10 @@ class Leaf(object):
         file_content = _get_data_from_url(self.url)
         if not file_content.status_code == 200:
             file_error_msg = "Could not fetch remote file: {}; {}".format(
-                self.url,
-                file_content.status_code
+                self.url, file_content.status_code
             )
-            logger.error(
-                file_error_msg
-            )
-            file_content = json.dumps([{
-                "url": self.url,
-                "error": file_error_msg
-            }])
+            click.ClickException(file_error_msg)
+            # file_content = json.dumps([{"url": self.url, "error": file_error_msg}])
         else:
             file_content = file_content.content
 
@@ -181,10 +206,7 @@ class Leaf(object):
         else:
             logger.error(f"data file format {self.format} is not supported!")
 
-        self.downloaded = {
-            "data": data,
-            "content": file_content
-        }
+        self.downloaded = {"data": data, "content": file_content}
 
     @property
     def data(self):
@@ -219,5 +241,5 @@ class Leaf(object):
             self.herb.id,
             self.leaf_meta_json.get("size"),
             self.path,
-            self.metadata()
+            self.metadata(),
         )
